﻿(function () {
    var app = angular.module("wttAdmin", []);

    app.config(['$httpProvider', function ($httpProvider) {

        $httpProvider.interceptors.push(["$q", function ($q) {
            return {
                request: function (config) {
                    if (config.url.indexOf("api.textmetrics.com") > 0) {
                        config.headers = config.headers || {};
                        config.headers.Authorization = 'Bearer ' + localStorage.getItem('wtt_token');
                        config.headers.WttSource = 'EPiServer';
                    }
                    return config;
                },
                response: function (response) {
                    return response || $q.when(response);
                }
            };
        }]);
    }]);

    // Register the controller
    app.controller('WttDashboardController',
        function ($scope, $http, $q, webtexttoolResource, documentTypeService) {

            $scope.wttTemplates = [
                {
                    url: '/EPiServer/Textmetrics/ClientResources/Partials/wtt-account.html'
                },
                {
                    url: '/EPiServer/Textmetrics/ClientResources/Partials/wtt-login.html'
                }
            ];

            $scope.error = null;
            $scope.message = "Invalid Email or Password";
            $scope.loading = false;
            $scope.logo = "/EPiServer/Textmetrics/ClientResources/Images/tm_logo.png";
            $scope.promiseMessage = "Please wait...";
            var wttApiBaseUrl = "https://api.textmetrics.com/";
            $scope.WttAppUrl = "https://app.textmetrics.com/#/";
            $scope.apiKey = {
                value: ""
            };

            $scope.loginModel = {
                UserName: "",
                Password: "",
                RememberMe: true
            };

            var getData = function (url) {
                var deffered = $q.defer();

                $http.get(url,
                    {
                        withCredentials: true
                    })
                    .success(function (data, status, headers, config) {
                        deffered.resolve(data);
                    }).error(function (error, status) { // called asynchronously if an error occurs
                        deffered.reject(error.Message);
                    });
                return deffered.promise;
            };

            var postData = function (url, dataIn) {
                var deffered = $q.defer();

                $http.post(url,
                    dataIn,
                    {
                        withCredentials: true
                    })
                    .success(function (data, status, headers, config) {
                        deffered.resolve(data);
                    }).error(function (error, status) { // called asynchronously if an error occurs
                        deffered.reject(error.Message);
                        $scope.error = error.Message;
                        $scope.loading = false;
                    });
                return deffered.promise;
            };

            var saveDashboardData = function (data) {
                var deffered = $q.defer();

                jQuery.ajax({
                    url: '/WTTSettings/Post',
                    type: 'POST',
                    dataType: 'json',
                    data: data,
                    success: function (result) {
                        deffered.resolve(result);
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        console.log(JSON.stringify(jqXHR));
                        console.log("AJAX error: " + textStatus + ' : ' + errorThrown);
                    }
                });

                return deffered.promise;
            };

            $scope.saveApiKey = function () {
                var data = {
                    apiKey: $scope.apiKey.value
                };

                localStorage.setItem('wtt_token', $scope.apiKey.value);

                saveDashboardData(data).then(function () {
                    window.location.reload();
                });
            };

            $scope.login = function () {
                $scope.loading = true;

                postData(wttApiBaseUrl + "user/login", $scope.loginModel).then(function (response) {
                    var data = {
                        accessToken: response.access_token
                    };

                    if (response) {
                        localStorage.setItem('wtt_token', response.access_token);

                        saveDashboardData(data).then(function () {
                            window.location.reload();
                        });
                    } else {
                        $scope.loading = false;
                        $scope.error = $scope.message;
                    }

                });
            };

            $scope.logout = function () {

                $scope.loading = true;

                getData(wttApiBaseUrl + "user/logout").then(function () {
                    localStorage.removeItem('wtt_token');

                    var data = {
                        apiKey: "",
                        accessToken: ""
                    };

                    saveDashboardData(data).then(function () {
                        window.location.reload();
                    });
                });
            };

            var getLightDocumentTypesList = function (lang, showHidden) {
                var deffered = $q.defer();

                documentTypeService.getDocumentTypes($scope.userInfo.DefaultLanguageCode).then(function (templates) {
                    var data = {
                        apiKey: $scope.apiKey2,
                        accessToken: $scope.authCode2,
                        docTypesTemplates: JSON.stringify(templates)
                    };

                    saveDashboardData(data).then(function () {
                        deffered.resolve();
                    });

                    return deffered.promise;
                });
            };

            function init() {
                webtexttoolResource.getWttData().then(function (response) {

                    $scope.apiKey2 = response.data.ApiKey;
                    $scope.authCode2 = response.data.AccessToken;

                    if (localStorage.getItem('wtt_token') === null || localStorage.getItem('wtt_token') === "") {
                        if ($scope.authCode2 !== '' && $scope.authCode2 !== null) {
                            localStorage.setItem('wtt_token', $scope.authCode2);
                            window.location.reload();
                        };

                        if ($scope.apiKey2 !== '' && $scope.apiKey2 !== null) {
                            localStorage.setItem('wtt_token', $scope.apiKey2);
                            window.location.reload();
                        };
                    }
                });

                getData(wttApiBaseUrl + "user/authenticated").then(function (result) {
                    $scope.wttAuth = result;

                    if (result) {
                        $scope.accountPromise = getData(wttApiBaseUrl + "user/info").then(function (userInfo) {
                            $scope.userInfo = userInfo;
                            getLightDocumentTypesList();

                            $scope.displayTrialDays = function (days) {
                                if (days <= 0) return "0";
                                return days;
                            };
                        });
                    }
                });
            }

            init();
        });

    app.factory("documentTypeService",
        function (httpService) {
            var WttApiBaseUrl = "https://api.textmetrics.com/";

            var getDocumentTypes = function (lang, showHidden) {
                var dataUrl = "/project/GetLightDocumentTypes";
                if (lang) {
                    dataUrl = dataUrl + "?lang=" + lang;
                }
                if (showHidden != undefined) {
                    dataUrl = dataUrl + "&showHidden=" + showHidden;
                }

                return httpService.getData(WttApiBaseUrl + dataUrl);
            };

            return {
                getDocumentTypes: getDocumentTypes,
            };
        }
    );

    app.factory("httpService", ['$http', '$q',
        function ($http, $q) {

            var getData = function (url) {

                var deffered = $q.defer();

                // Make a get request
                $http.get(url, {
                    withCredentials: true
                })
                    .success(function (data, status, headers, config) {
                        deffered.resolve(data);
                    }).error(function (error, status) {

                        if (status === 401) {
                            return;
                        }

                        if (status === 500) {
                            console.error('Please contact us at <a href="mailto:support@textmetrics.com">support@textmetrics.com</a> Something went wrong while retrieving the data!');
                            return;
                        }

                        deffered.reject(error.Message);
                    });

                return deffered.promise;
            };


            var postData = function (url, dataIn) {

                var deffered = $q.defer();

                // Make a post request
                $http.post(url, dataIn, {
                    withCredentials: true
                })
                    .success(function (data, status, headers, config) {
                        deffered.resolve(data);
                    }).error(function (error, status) {

                        if (status === 400) {
                            console.error('Your request params are empty or invalid! Bad Request');
                            return;
                        }

                        if (status === 401) {
                            console.error(error.Message + 'Unauthorized');
                            return;
                        }

                        if (status === 403) {
                            console.error("Please contact us at <a href=\"mailto:support@textmetrics.com\">support@textmetrics.com</a> Access denied (" + status + ")");
                            return;
                        }

                        if (status === 500) {
                            console.error('Please contact us at <a href="mailto:support@textmetrics.com">support@textmetrics.com</a> Something went wrong!');

                            deffered.reject(error.Message);
                        }

                        deffered.reject(error.Message);
                    });

                return deffered.promise;

            };

            return {
                getData: getData,
                postData: postData
            }
        }
    ]);

    app.factory("webtexttoolResource", function ($http) {
        return {
            getWttData: function () {
                return $http.get("/WTTSettings/Get");
            }
        };
    });
})();